<?php
/*-----------------------------------------------------------------------------
                     ADMIN UPLOAD UTILITY - A phpBB Add-On
  ----------------------------------------------------------------------------
    Please read the file README.TXT for licensing and copyright information.
  ----------------------------------------------------------------------------
    class_auu.php - Admin Upload Utility Class File
    File Version: 2.2.1
    Begun: August 29, 2004
    Last Modified: Monday, January 24, 2005
-----------------------------------------------------------------------------*/

// AUU_Config - Used to process, store, and access configuration data.
class AUU_Config
{
	var $cfg = array();			// Configuration data.
	var $options = array();		// Submitted form options and data.
	var $int_checks = array();	// Fields to be run through intval().
	var $module;				// Lowercase module internal name.
	var $upper;					// Uppercase module internal name.
	var $loaded;				// Number of loaded module cfg files.
	
/*  Constructor: AUU_Config( void )
	Sets up the options array with request and form data.
*/
	function AUU_Config()
	{
		// Get some URI Request Data.
		$this->options['mode'] = ( isset($_REQUEST['mode']) ) ? $_REQUEST['mode'] : '';
		$this->options['module_name'] = ( isset($_REQUEST['type']) ) ? $_REQUEST['type'] : '';

		// Get some form submissions options.
		$this->options['auto_add'] = ( !empty($_REQUEST['autoadd']) ) ? true: false;
		$this->options['delete'] = ( isset($_REQUEST['objects']) ) ? $_REQUEST['objects']: false;
		$this->options['del_tpls'] = ( isset($_REQUEST['objects']) ) ? array() : false;
		$this->options['delete_all'] = ( !empty($_REQUEST['delete_all']) ) ? true: false;
		$this->options['delete_db'] = ( !empty($_REQUEST['delete_db']) ) ? true: false;

		$this->loaded = 0;

		// Check what we have for errors.
		$this->do_sanity_checks();
	}

/*  Function: void set_module_name( string $name )
	Creates shortcuts for referencing module names in main code.

	$name - The current module internal name, as pulled from a cfg file.
*/
	function set_module_name($name)
	{
		$this->module = $name;
		$this->upper = strtoupper($this->module);
	}

/*  Function: void get_config_file( [ string $grab ] )
	Creates shortcuts for referencing module names in main code.

	$grab - Either a module name, such as "smilies" or "ranks," or "all".
				Defaults to "all".  Names will cause a single cfg file to be
				loaded, and "all" will load every available cfg file. cfg
				files must be in the AUU_PATH directory.
*/
	function get_config_file($grab = 'all')
	{
		if( $grab == 'all' )
		{
			$modules_dir = @opendir(AUU_PATH);
			while( $module_file = @readdir($modules_dir) )
			{
				if( preg_match('/\.cfg$/', $module_file) )
				{
					$this->parse_cfg_file(@file(AUU_PATH . $module_file));
				}
			}
			@closedir($modules_dir);
		}
		else
		{
			if( file_exists(AUU_PATH . $grab . '.cfg') )
			{
				$this->parse_cfg_file(@file(AUU_PATH . $grab . '.cfg'), true);
			}
		}
	}

/*  Function: void parse_cfg_file( array $cfg_file[, boolean $single] )
	Reads configuration data from a text file. The data is parsed into an array
	after being checked for integrity. The data array is added to $this->cfg.
	Details of configuration file layout are in the hack documentation.

	$cfg_file - Contains the contents of a Admin Upload Utility
					configuration file.
	$single   - If true, $this->cfg will be given value of a single data
					array. If false, $this->cfg will be holding several data
					arrays in a second dimension, such as
					$this->cfg['smilies'], $this->cfg['medals'].
*/
	function parse_cfg_file($cfg_file, $single = false)
	{
		global $board_config, $phpbb_root_path;

		// Is there anything in the file array?
		if( empty($cfg_file) || !is_array($cfg_file) )
		{
			return;
		}

		// Build search/replace arrays with phpBB supplied values.
		// Paths in config files will be parsed with these using str_replace
		$search = array('{PHPBB_ROOT}');
		$replace = array($phpbb_root_path);
		foreach($board_config as $k=>$v)
		{
			$search[] = '{' . strtoupper($k) . '}';
			$replace[] = $v;
		}

		$arr = array();
		foreach($cfg_file as $line)
		{
			$name = $val = '';
			$line = trim(htmlspecialchars(stripslashes($line)));
			if( empty($line) )
			{
				continue; // Empty line, skip to the next line
			}
			list($name, $val) = explode('=', $line);
			$name = trim($name);
			$val = trim($val);
			if( empty($name) )
			{
				continue; // No name in this pair, skip to next line
			}

			if( $name == 'internal_name' && array_key_exists($val, $this->cfg) )
			{
				continue; // cfg file with this name seems to be loaded already.
			}

			switch($name)
			{
				case 'allowed_types':
					// Multiple allowed_types lines are allowed in cfg files, and
					// will create an array inside the data array.
					$temp = explode(',', $val);
					if( is_array($temp) )
					{
						foreach($temp as $k=>$v)
						{
							$temp[$k] = trim($v);
						}
						$arr[$name][$temp[0]] = $temp[1];
						unset($temp);
					}
					break;
				case 'file_path':
				case 'db_path':
					// Parse {VARS} in paths
					$val = str_replace($search, $replace, $val);
					// Add trailing slash
					$val .= ( !empty($val) && ($val{strlen($val)-1} != '/') ) ? '/' : '';
					// No break here!
				default:
					if( in_array($name, $this->int_checks) )
					{
						$val = intval($val); // Make sure these are integers.
					}
					$arr[$name] = $val;
			}
		}

		if( !empty($arr) && !empty($arr['internal_name']) )
		{
			// Get max_filesize in KB for displaying to users.
			if( !empty($arr['max_filesize']) )
			{
				$arr['max_filesize_kb'] = intval(round($arr['max_filesize'] / 1024));
			}
			else
			{
				$arr['max_filesize_kb'] = 0;
			}

			// Add completed data array to $this->cfg.
			if( $single )
			{
				$this->cfg = $arr;
			}
			else
			{
				$this->cfg[$arr['internal_name']] = $arr;
			}
			$this->loaded++;
		}
	}

/*  Function: void do_sanity_checks( void )
	Rule #1 of programming: Don't Trust User Input.
*/
	function do_sanity_checks()
	{
		$this->options['mode'] = trim(htmlspecialchars(stripslashes($this->options['mode'])));
		$this->options['module_name'] = trim(htmlspecialchars(stripslashes($this->options['module_name'])));

		// This is a tricky form input, with SQL injection holes to plug.
		if( is_array($this->options['delete']) )
		{
			foreach($this->options['delete'] as $key=>$val)
			{
				// Split path, file information
				list($tpl, $file) = explode('+-+', $val, 2);
				// Limit delete array to file only.
				$this->options['delete'][$key] = trim(htmlspecialchars(stripslashes($file)));
				// Create matching path entry
				$this->options['del_tpls'][$key] = trim(htmlspecialchars(stripslashes($tpl)));
			}
		}
		else
		{
			$this->options['delete'] = false;
		}

		// Config file entries that should always be integers. We'll actually
		// check these when parsing the cfg files.
		$this->int_checks = array('max_width', 'max_height', 'max_filesize', 'table_columns', 'min_width', 'min_height', 'tpl', 'db');
	}

/*  Function: void check_names( void )
	Prints an error if no module name has been supplied from a URL or cfg file.
*/
	function check_names()
	{
		if( empty($this->options['module_name']) )
		{
			// No module name? We'sa ina trouble!
			message_die(GENERAL_MESSAGE, 'UPGEN_NO_MODULE');
		}
		elseif( empty($this->cfg[$this->module]['internal_name']) )
		{
			// Missing some configuration data for this module. Aiyah!
			message_die(GENERAL_MESSAGE, 'UPGEN_NO_CFG');
		}
	}

}


// AUU_General - Holds basic functions used by script occasionally.
class AUU_General 
{
	var $error_msg;			// Compound error message.
	var $module;			// Lowercase module internal name.
	var $upper;				// Uppercase module internal name.
	var $pattern;			// Regular expression pattern.
	var $files_present;		// Switch to indicate files are currently uploaded.

/*  Constructor: AUU_General( object $config )
	Gets the language files for the general uploader and the specific module.

	$config - An instance of the AUU_Config class.
*/
	function AUU_General($config)
	{
		$this->error_msg = '';
		$this->module = $config->module;
		$this->upper = $config->upper;
		$this->files_present = TRUE;
	}

/*  Function: void get_lang_files( void )
	Gets the language files for the general uploader and the specific module.
*/
	function get_lang_files()
	{
		global $lang, $phpbb_root_path, $board_config, $auu;

		$file_base = $phpbb_root_path . 'language/lang_' . $board_config['default_lang'] . '/lang_upload';

		require_once($file_base . PHPEX);

		// Pull module language file if one exists.
		if( @file_exists($file_base . '_' . $this->module . PHPEX) )
		{
			require_once($file_base . '_' . $this->module . PHPEX);
		}
	}

/*  Function: void build_error_msg
					( string $str, [ mixed $splice1, ... mixed $spliceN ] )
	Creates a compound error message as script proceeds.

	$str     - An error message, an index of the $lang array, or a partial index
				that does not contain the module name.  In the latter case, the module name will be added automatically.
	$splice  - A piece of data to be inserted into $str using vsprintf().  You
				can send as many splices as you wish.
*/
	function build_error_msg($str)
	{
		global $lang;

		$this->error_msg .= ( !empty($this->error_msg) ) ? '<br />' : '';
		if( isset($lang[$str]) )
		{
			$str = $lang[$str];
		}
		elseif( isset($lang[$this->upper . '_' . $str]) )
		{
			$str = $lang[$this->upper . '_' . $str];
		}

		if( func_num_args() > 1 )
		{
			$splices = func_get_args();
			array_shift($splices);
			$str = vsprintf($str, $splices);
		}
		
		$this->error_msg .= $str;
	}

/*  Function: void form_blazing_pattern( array $basis )
	Builds a pattern from allowed file types for use in regular expressions.
	after being checked for integrity. The data array is added to $this->cfg.
	Details of configuration file layout are in the hack documentation.

	$basis - An array of allowed file types, from a cfg file.
*/
	function form_blazing_pattern($basis)
	{
		$this->pattern = '';
		foreach($basis as $key=>$val)
		{
			$this->pattern .= $key . '|';
		}
		$this->pattern = substr($this->pattern, 0, -1);
		if( empty($this->pattern) )
		{
			message_die(GENERAL_MESSAGE, 'UPGEN_NO_ALLOWED_TYPES');
		}
	}

/*  Function: array get_templates_list( void )
	Returns an array of folder names from the templates/ directory.
*/
	function get_templates_list()
	{
		global $phpbb_root_path;
		$templates_dir = opendir($phpbb_root_path . 'templates');
		$templates = array();
		while ( false !== ($plate = readdir($templates_dir)) )
		{
			$plate_dir = $phpbb_root_path . 'templates/' . $plate . '/';
			if( !in_array($plate, array('.', '..')) && is_dir($plate_dir) && file_exists($plate_dir . $plate . '.cfg') )
			{
				// Seems to be a directory with an existing tpl config file.
				$templates[] = $plate;
			}
		}
		closedir($templates_dir);
		return $templates;
	}

/*  Function: void display_files( array $config )
	Displays uploaded files, based on template dependancy.

	$config - An array of module config values.
*/
	function display_files($config)
	{
		if( $config['tpl'] )
		{
			// Images are template dependent. Find them for each template.
			$templates = $this->get_templates_list();
			foreach($templates as $val)
			{
				$this->show_uploaded($config, $val);
			}
		}
		else
		{
			// Not template dependent.
			$this->show_uploaded($config);
		}
	}

/*  Function: string build_tpl_path( string $file_path , string $tpl  )
	Returns a file_path for a location inside a template's folder.

	$file_path - An file_path configuration setting.
	$tpl       - Name of the current template, when template dependent.
*/
	function build_tpl_path($file_path, $tpl)
	{
		if( !empty($tpl) )
		{
			$file_path = str_replace('{TEMPLATE_PATH}', 'templates/' . $tpl . '/', $file_path);
		}
		return $file_path;
	}

/*  Function: void show_uploaded( array $cfg [, string $template ] )
	Searches for and displays uploaded files found in the file system.

	$cfg - An array of module config values.
	$template - Name of the current template, when template dependent.
*/
	function show_uploaded($cfg, $template_name = '')
	{
		global $lang, $template;

		$file_path = $this->build_tpl_path($cfg['file_path'], $template_name);

		// Show uploaded objects for deleting
		$objects_dir = opendir($file_path);
		$objects_files = array();
		$this->form_blazing_pattern($cfg['allowed_types']);
		while ( false !== ($object = readdir($objects_dir)) )
		{
			if( preg_match('/(.*)\.(' . $this->pattern . ')$/i', $object) )
			{
				$objects_files[] = $object;
			}
		}
		closedir($objects_dir);
		$objects_count = count($objects_files);

		$template->assign_block_vars('file_display', array(
			'TEMPLATE' => $template_name,
			'PATH' => $file_path
		));

		if( $objects_count )
		{
			sort($objects_files);

			if( !empty($template_name) )
			{
				$template->assign_block_vars('file_display.tpl_header', array());
			}

			$objects_rows = ceil($objects_count/$cfg['table_columns']);
			$jj=0;
			for($ii=0; $ii<$objects_rows; $ii++)
			{
				$first_object = $jj;
				if( $objects_rows == 1 )
				{
					$last_object = $objects_count;
				}
				else
				{
					$last_object = $jj + $cfg['table_columns'];
				}
				$template->assign_block_vars('file_display.uploaded_row', array());
				for($jj=$first_object; $jj<$last_object; $jj++)
				{
					if( $objects_files[$jj] )
					{
						$template->assign_block_vars('file_display.uploaded_row.uploaded_cell', array(
							'NAME' => $objects_files[$jj]
						));
					}
					else
					{
						$template->assign_block_vars('file_display.uploaded_row.empty_cell', array());
					}
				}
			}
		}
		else
		{
			$this->files_present = FALSE;
			$template->assign_block_vars('file_display.none_uploaded', array(
				'L_SORRY_NONE' => sprintf($lang['AUUMODULE_SORRY_NONE'], $cfg['object_name'], $cfg['object_type_display'])
			));
		}
	}

/*  Function: mixed run_module_function( string $func_name [, mixed $param ] )
	Runs a module specific function, if that function exists.  Returns the
	return value of the function, or false if the function does not exist.

	$func_name - The name of a function in a module.
	$param     - A parameter for the module function.  This is optional.
*/
	function run_module_function($func_name)
	{
		if( function_exists($func_name) )
		{
			if( func_num_args() > 1 )
			{
				// A parameter was passed. We only use second argument for this
				$params = func_get_args();
				array_shift($params);
				$arg = array_shift($params);
				unset($params);
				return $func_name($arg);
			}
			return $func_name();
		}

		return FALSE;
	}
}



// AUU_Uploader - Handles main uploading tasks that are identical for all objects.
class AUU_Uploader 
{
	var $new_filename;			// Filename of uploaded file.

	// Info from form fields.
	var	$upload_name;
	var	$tmp_name;
	var	$filesize;
	var	$filetype;
	var	$image_name;
	var	$default_name;
	var	$tpl;

/*  Constructor: AUU_Uploader( void )
*/
	function AUU_Uploader()
	{
		$this->new_filename = '';
		$this->deleted_db = false;
	}

/*  Function: void run_preprocessor( void )
	Performs any processing need, by modules or this class, before uploading
	a file.
*/
	function run_preprocessor()
	{
		global $auu, $gen;

		// Get information from upload form.
		$this->upload_name = ( !empty($_FILES['uploadfile']['name']) ) ? $_FILES['uploadfile']['name'] : '';
		$this->tmp_name = ( $_FILES['uploadfile']['tmp_name'] != 'none') ? $_FILES['uploadfile']['tmp_name'] : '';
		$this->filesize = ( !empty($_FILES['uploadfile']['size']) ) ? $_FILES['uploadfile']['size'] : 0;
		$this->filetype = ( !empty($_FILES['uploadfile']['type']) ) ? $_FILES['uploadfile']['type'] : '';
		$this->image_name = ( !empty($_POST['filename']) ) ? trim(htmlspecialchars($_POST['filename'])) : '';
		$this->default_name = ( isset($_POST['defaultname']) ) ? true : false;
		$this->tpl = ( !empty($_POST['style']) ) ? trim(htmlspecialchars(stripslashes($_POST['style']))) : '';

		return $gen->run_module_function('do_before_upload');
	}

/*  Function: void run_postprocessor( void )
	Performs any processing need, by modules or this class, after uploading
	a file.
*/
	function run_postprocessor()
	{
		global $gen;

		// Who knows, I might need this one day...
		return $gen->run_module_function('do_after_upload');
	}

/*  Function: void check_object_type( string $type )
	Checks the file type of an object against allowed types for a module.

	$type - An object's file type, pulled from the file.
*/
	function check_object_type($type)
	{
		global $auu, $gen;

		$ext = false;
		if( isset($auu->cfg['allowed_types'][$type]))
		{
			$ext = '.' . $auu->cfg['allowed_types'][$type];
		}
		else
		{
			$gen->build_error_msg('AUUMODULE_FILETYPE', implode(', ', array_keys($auu->cfg['allowed_types'])));
		}

		return $ext;
	}

/*  Function: void upload_object( void )
	Upload an object to the server.
*/
	function upload_object()
	{
		global $board_config, $db, $lang, $phpbb_root_path, $auu, $gen;

		if ( !$this->upload_name )
		{
			$gen->build_error_msg('UPGEN_SELECT_FILE');
			return false;
		}

		if( !($this->filesize > 0) )
		{
			$gen->build_error_msg($lang['UPGEN_FILESIZE_LOW']);
			return false;
		}
		elseif( $auu->cfg['max_filesize'] > 0 && !($this->filesize <= $auu->cfg['max_filesize']) )
		{
			$gen->build_error_msg($lang['UPGEN_FILESIZE'], $auu->cfg['max_filesize_kb']);
			return false;
		}

		preg_match('#' . $auu->cfg['object_type'] . '\/[x\-]*([a-z]+)#', $this->filetype, $this->filetype);
		$size = getimagesize($this->tmp_name);

		if( $auu->cfg['max_width'] > 0 && ($size[0] > $auu->cfg['max_width']) )
		{
			$gen->build_error_msg($lang['UPGEN_MAX_WIDTH'], $auu->cfg['max_width']);
			return false;
		}

		if( $auu->cfg['max_height'] > 0 && ($size[1] > $auu->cfg['max_height']) )
		{
			$gen->build_error_msg($lang['UPGEN_MAX_HEIGHT'], $auu->cfg['max_height']);
			return false;
		}

		if( $auu->cfg['min_width'] > 0 && !($size[0] > $auu->cfg['min_width']) )
		{
			$gen->build_error_msg($lang['UPGEN_MIN_WIDTH'], $auu->cfg['min_width']);
			return false;
		}

		if( $auu->cfg['min_height'] > 0 && !($size[1] > $auu->cfg['min_height']) )
		{
			$gen->build_error_msg($lang['UPGEN_MIN_HEIGHT'], $auu->cfg['min_height']);
			return false;
		}

		if( empty($this->tpl) && $this->tpl != 0 )
		{
			$gen->build_error_msg($lang['UPGEN_NO_TEMPLATE']);
			return false;
		}
		elseif( $this->tpl != 0 )
		{
			$tpl_path = $phpbb_root_path . 'templates/' . $this->tpl . '/';
			if ( !file_exists($tpl_path) && is_dir($tpl_path) )
			{
				$gen->build_error_msg($lang['UPGEN_TEMPLATE_NF']);
				return false;
			}
		}

		if ( !($imgtype = $this->check_object_type($this->filetype[1])) )
		{
			return false;
		}

		$this->new_filename = '';
		if( !$this->default_name )
		{
			if( empty($this->image_name) )
			{
				$gen->build_error_msg('UPGEN_NO_NAME');
				return false;
			}
			$this->new_filename = $this->image_name;
		}
		else
		{
			$gen->form_blazing_pattern($auu->cfg['allowed_types']);
			preg_match('/(.*)\.(' . $gen->pattern . ')$/i', $this->upload_name, $filename);
			$this->new_filename = $filename[1];
		}
		$this->new_filename = str_replace(' ', '_', urldecode($this->new_filename)) . $imgtype;
		if( empty($this->new_filename) )
		{
			$gen->build_error_msg('UPGEN_FILENAME_FAILED');
			return false;
		}

		$file_path = $gen->build_tpl_path($auu->cfg['file_path'], $this->tpl);
		if( file_exists($file_path . $this->new_filename) )
		{
			$gen->build_error_msg('AUUMODULE_FILE_ALREADY', $auu->cfg['object_name']);
			return false;
		}

		$move_file = ( @get_cfg_var('open_basedir') != '' ) ? 'move_uploaded_file': 'copy';

		$file_moved = $move_file($this->tmp_name, $file_path . $this->new_filename);

		if( $file_moved )
		{
			@chmod($file_path . $this->new_filename, 0777);

			$gen->build_error_msg('UPGEN_UPLOAD_SUCCESFUL');
			return true;
		}

		$gen->build_error_msg('UPGEN_UPLOAD_FAILED');
		return false;
	}

/*  Function: void add_database_entry( void )
	Wrapper method for calling the module-specific auto_add_object()
*/
	function add_database_entry()
	{
		if( function_exists('auto_add_object') )
		{
			global $gen, $auu;

			if( !$gen->run_module_function('auto_add_object', $this->new_filename) )
			{
				$gen->build_error_msg('AUUMODULE_ADD_FAILED', $auu->cfg['object_name']);
			}
			else
			{
				$gen->build_error_msg('AUUMODULE_ADD_SUCCESSFUL', $auu->cfg['object_name']);
			}
		}
	}

/*  Function: void deleter( string $path, string $name )
	Tries to delete a file and, if desired, it's database entry.

	$path - Path to the file.
	$name - Name of the file.
*/
	function deleter($path, $name)
	{
		global $gen, $auu;

		if( file_exists($path . $name) )
		{
			if( @unlink($path . $name) )
			{
				$gen->build_error_msg('UPGEN_DELETE_SUCCESSFUL', $path . $name);
				if( function_exists('delete_db_object') && $auu->options['delete_db'] )
				{
					$gen->build_error_msg( $gen->run_module_function('delete_db_object', $name), $auu->cfg['display_name'] );
				}
			}
			else
			{
				$gen->build_error_msg('UPGEN_DELETE_FAILED', $path . $name);
			} // unlink
		}
	}
}

?>