<?php
/*-----------------------------------------------------------------------------
        ADMIN UPLOAD UTILITY - GALLERY AVATARS MODULE - A phpBB Add-On
  ----------------------------------------------------------------------------
    Please read the file README.TXT for licensing and copyright information.
  ----------------------------------------------------------------------------
    module_avagallery.php - Gallery Avatars Module File
    File Version: 1.0.0
    Begun: Monday, December 06, 2004
    Last Modified: Tuesday, December 07, 2004
-----------------------------------------------------------------------------*/

if ( !defined('IN_PHPBB') )
{
	die('Hacking attempt');
}

if( !empty($setmodules) )
{
	$module['General']['  '] = $filename . '?type=avagallery';
	return;
}

// Change some classes to extended versions.  This is needed to correctly
// display and delete gallery avatars.

$gen    = new GalleryDisplayer($auu);
$upload = new GalleryDeleter();


/*  Function: boolean do_before_upload( void )
	Completes any necessary tasks that must be done before a file is actually
	uploaded.
*/
function do_before_upload()
{
	global $auu, $gen;

	// Get additional form input for destination gallery folder.
	$gallery_subcat = ( isset($_POST['gallery_subcat']) ) ? trim(htmlspecialchars(stripslashes($_POST['gallery_subcat']))): '';

	if( empty($gallery_subcat) )
	{
		$gen->build_error_msg('AVAGALLERY_NO_PATH');
		return NULL;
	}
	
	$auu->cfg['file_path'] .= $gallery_subcat . '/';

	return true;
}

/*  Function: void do_after_upload( void )
	Completes any necessary tasks that must be done after a file is actually
	uploaded.
*/
function do_after_upload()
{
	global $auu;

	// Get additional form input for destination gallery folder.
	$gallery_subcat = ( isset($_POST['gallery_subcat']) ) ? trim(htmlspecialchars(stripslashes($_POST['gallery_subcat']))): '';
	$trim_length = -(strlen($gallery_subcat) +1);
	$auu->cfg['file_path'] = substr($auu->cfg['file_path'], 0, $trim_length);
}

/*  Function: void assign_module_tpl( void )
	Sets up any template features specific to this module.  This may include new
	template variables and switches.
*/
function assign_module_tpl()
{
	global $auu, $phpbb_root_path, $board_config, $template, $lang;

	// Get avatar gallery categories.
	$categories = array();
	// Set directory names to be excluded.
	$exclude = array('.', '..');
	$path = $phpbb_root_path . $board_config['avatar_gallery_path'];
	$path = ( strrpos($path, '/') !== strlen($path) ) ? $path . '/': $path;
	$avatar_dir = opendir($path);
	while( false !== ($object = readdir($avatar_dir)) )
	{
		if( !in_array($object, $exclude) && is_dir($path . $object) )
		{
			$categories[] = $object;
		}
	}
	closedir($avatar_dir);
	sort($categories);


	if( !count($categories) )
	{
		$select = sprintf($lang['AVAGALLERY_NOCATS'], $path);
	}
	else
	{
		// Create selection list from categories.
		$select = '<select name="gallery_subcat">';
		foreach($categories as $cat)
		{
			$select .= '<option value="'. $cat .'">'. ucfirst($cat) .'</option>';
		}
		$select .= '</select>';
	}

	$template->assign_block_vars('switch_uploads.module_avagallery', array());
	$template->assign_vars(array(
		'AVA_GALLERY_DESC' => $lang['AVAGALLERY_DESC'],
		'AVA_GALLERY_SELECT' => $select,
		'MODULE_COPYRIGHTS' => $auu->cfg['display_name'] . ' Module &copy; 2004 Thoul'
	));
}

class GalleryDisplayer extends AUU_General 
{
/*  Function: void display_files( array $cfg )
	Searches for and displays uploaded files found in the file system.
	This is a replacement for the default Admin Upload Utility's AUU General >
	display_files and show_uploaded methods.  It's been reworked to display
	avatar gallery categories and images.

	$cfg - An array of module config values.
*/
	function display_files($cfg)
	{
		global $auu, $phpbb_root_path, $board_config, $template, $lang;

		// Set directory names to be excluded.
		$exclude = array('.', '..');
		// Get avatar gallery categories.
		$categories = array();
		$path = $phpbb_root_path . $board_config['avatar_gallery_path'];
		$path = ( strrpos($path, '/') !== strlen($path) ) ? $path . '/': $path;
		$avatar_dir = opendir($path);
		while( false !== ($cat = readdir($avatar_dir)) )
		{
			if( !in_array($cat, $exclude) && is_dir($path . $cat) )
			{
				$categories[$cat] = array();
				$sub_dir = @opendir($path . $cat);
				while( false !== ($ava_file = readdir($sub_dir)) )
				{
					if( preg_match('/(\.gif$|\.png$|\.jpg|\.jpeg)$/is', $ava_file) )
					{
						$categories[$cat][] = $ava_file; 
					}
				}
				closedir($sub_dir);
			}
		}
		closedir($avatar_dir);
		ksort($categories);

		foreach($categories as $cat=>$cat_images)
		{
			$cat_count = count($cat_images);

			$template->assign_block_vars('file_display', array(
				'TEMPLATE' => ucfirst($cat),
				'PATH' => $path . $cat . '/'
			));

			$template->assign_block_vars('file_display.tpl_header', array());
			if( $cat_count )
			{
				sort($cat_images);


				$objects_rows = ceil($cat_count/$cfg['table_columns']);
				$jj=0;
				for($ii=0; $ii<$objects_rows; $ii++)
				{
					$first_object = $jj;
					if( $objects_rows == 1 )
					{
						$last_object = $cat_count;
					}
					else
					{
						$last_object = $jj + $cfg['table_columns'];
					}
					$template->assign_block_vars('file_display.uploaded_row', array());
					for($jj=$first_object; $jj<$last_object; $jj++)
					{
						if( $cat_images[$jj] )
						{
							$template->assign_block_vars('file_display.uploaded_row.uploaded_cell', array(
								'NAME' => $cat_images[$jj]
							));
						}
						else
						{
							$template->assign_block_vars('file_display.uploaded_row.empty_cell', array());
						}
					}
				}
			}
			else
			{
				$this->files_present = FALSE;
				$template->assign_block_vars('file_display.none_uploaded', array(
					'L_SORRY_NONE' => sprintf($lang['AUUMODULE_SORRY_NONE'], $cfg['object_name'], $cfg['object_type_display'])
				));
			}
		}
	}
}

class GalleryDeleter extends AUU_Uploader 
{
	var $partially_deleted_files;
	var $partially_deleted_paths;

/*  Constructor: void GalleryDeleter( void )
	Sets up extended class features.
*/
	function GalleryDeleter()
	{
		global $auu;
		$this->partially_deleted_files = $auu->options['delete'];
		$this->partially_deleted_paths = $auu->options['del_tpls'];
	}

/*  Function: void deleter( string $path, string $name )
	Tries to delete a file and, if desired, it's database entry.
	Extended for gallery avatars.

	$path - Path to the file.
	$name - Name of the file.
*/
	function deleter($path, $name)
	{
		global $gen, $auu;

		// Locate file's position in delete array.
		$key_loc = array_search($name, $this->partially_deleted_files);
		$subcat = $this->partially_deleted_paths[$key_loc] . '/';

		if( empty($subcat) )
		{
			$subcat = '';
		}

		if( file_exists($path . $subcat . $name) )
		{
			if( @unlink($path . $subcat . $name) )
			{
				$gen->build_error_msg('UPGEN_DELETE_SUCCESSFUL', $path . $subcat . $name);
				if( function_exists('delete_db_object') && $auu->options['delete_db'] )
				{
					$gen->build_error_msg( $gen->run_module_function('delete_db_object', $name), $auu->cfg['display_name'] );
				}
			}
			else
			{
				$gen->build_error_msg('UPGEN_DELETE_FAILED', $path . $subcat . $name);
			} // unlink

			// Remove file from partial arrays, to be sure it is not processed more than once.
			unset($this->partially_deleted_files[$key_loc]);
			unset($this->partially_deleted_paths[$key_loc]);
		}
	}
}

?>